<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagicCart\Cart;

use Psr\Log\LoggerInterface;
use Psr\Log\NullLogger;
use WPDesk\ShopMagic\Customer\CustomerFactory;
use WPDesk\ShopMagic\Database\Abstraction\DAO;
use WPDesk\ShopMagic\Database\Abstraction\DAO\Item;
use WPDesk\ShopMagic\Exception\CannotCreateGuestException;

final class CartFactory implements DAO\ItemFactory {
	const STATUS_NEW       = 'new';
	const STATUS_ACTIVE    = 'active';
	const STATUS_ABANDONED = 'abandoned';
	const STATUS_RECOVERED = 'recovered';
	const STATUS_ORDERED   = 'ordered';

	/** @var CustomerFactory */
	private $customer_factory;

	/** @var LoggerInterface */
	private $logger;

	public function __construct( CustomerFactory $customer_factory, LoggerInterface $logger = null ) {
		$this->customer_factory = $customer_factory;
		$this->logger = $logger ?? new NullLogger();
	}

	public function create_null(): Item {
		return apply_filters(
			'shopmagic/carts/create_cart',
			new ActiveCart(
				0,
				Cart::FRESH,
				$this->customer_factory->create_null(),
				new \DateTimeImmutable(),
				new \DateTimeImmutable(),
				[],
				[],
				[],
				0,
				0,
				0,
				'',
				get_woocommerce_currency()
			)
		);
	}

	public function create_item( array $data ): Item {
		if ( ! empty( $data['user_id'] ) ) {
			$customer = $this->customer_factory->create_from_user( new \WP_User( $data['user_id'] ) );
		} else {
			try {
				$customer = $this->customer_factory->create_from_guest_id( (int) $data['guest_id'] );
			} catch ( CannotCreateGuestException $e ) {
				$customer = $this->customer_factory->create_null();
			}
		}

		$status = $data['status'];

		if ( $status === Cart::ACTIVE || $status === Cart::FRESH ) {
			$cart_class = ActiveCart::class;
		} elseif ( $status === Cart::SUBMITTED ) {
			$cart_class = SubmittedCart::class;
		} elseif ( $status === Cart::ABANDONED ) {
			$cart_class = AbandonedCart::class;
		} elseif ( $status === Cart::RECOVERED || $status === Cart::ORDERED ) {
			$cart_class = OrderedCart::class;
		} else {
			$cart_class = NullCart::class;
			$this->logger->error(sprintf('Could not create cart for status: %s', $status));
		}

		$cart = new $cart_class(
			(int) $data['id'],
			$data['status'],
			$customer,
			new \DateTimeImmutable( $data['last_modified'] ),
			new \DateTimeImmutable( $data['created'] ),
			json_decode( $data['items'], true ),
			json_decode( $data['coupons'], true ),
			json_decode( $data['fees'], true ),
			(float) $data['shipping_tax_total'],
			(float) $data['shipping_total'],
			(float) $data['total'],
			$data['token'],
			$data['currency']
		);

		return apply_filters( 'shopmagic/carts/create_cart', $cart );
	}
}
