<?php

namespace WPDesk\ShopMagicCart\Frontend;

use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use Psr\Log\LoggerInterface;
use ShopMagicCartVendor\WPDesk\PluginBuilder\Plugin\Hookable;
use WPDesk\ShopMagic\Exception\CannotProvideItemException;
use WPDesk\ShopMagic\Placeholder\Helper\PlaceholderUTMBuilder;
use WPDesk\ShopMagicCart\Cart\AbandonedCart;
use WPDesk\ShopMagicCart\Cart\CartTable;

/**
 * Restore user's cart.
 */
final class CartRestore implements Hookable, LoggerAwareInterface {
	use LoggerAwareTrait;

	const ACTION_NAME = 'cart-restore';

	/** @var CartTable */
	private $cart_table;

	public function __construct( CartTable $cart_table, LoggerInterface $logger ) {
		$this->cart_table = $cart_table;
		$this->logger     = $logger;
	}

	public static function get_restore_url( string $cart_token ): string {
		return add_query_arg(
			[
				'action' => self::ACTION_NAME,
				'token'  => $cart_token,
				'hash'   => self::calculate_hash( $cart_token ),
			],
			wc_get_cart_url()
		);
	}

	private static function calculate_hash( string $token ): string {
		return md5( $token . AUTH_SALT );
	}

	/**
	 * @return void
	 * @internal
	 */
	public function cart_restore_callback() {
		//phpcs:disable WordPress.Security.NonceVerification.Recommended
		if ( isset( $_GET['action'] ) && $_GET['action'] === self::ACTION_NAME ) {
			$token           = isset( $_GET['token'] ) ? sanitize_key( $_GET['token'] ) : '';
			$calculated_hash = self::calculate_hash( $token );
			$get_hash        = isset( $_GET['hash'] ) ? sanitize_key( $_GET['hash'] ) : '';
			if ( $get_hash !== $calculated_hash ) {
				return;
			}
			$this->restore_cart( $token );

			$utm_builder        = new PlaceholderUTMBuilder();
			$cart_link_with_utm = $utm_builder->append_utm_parameters_to_uri( $_GET, wc_get_cart_url() );
			wp_safe_redirect( $cart_link_with_utm );

			exit;
		}
		//phpcs:enable
	}

	/** @return void */
	private function restore_cart( string $token ) {
		try {
			$cart = $this->cart_table->get_single_by_where( [ 'token' => $token ] );
		} catch ( CannotProvideItemException $e ) {
			return;
		}
		if ( $cart instanceof AbandonedCart && WC()->cart instanceof \WC_Cart && count( $cart->get_items() ) !== 0 ) {
			$cart->append_to_wc_cart( WC()->cart );
		}
	}

	/** @return void */
	public function hooks() {
		add_action( 'wp', [ $this, 'cart_restore_callback' ] );
	}
}
